// ;-*-C++-*- *  Time-stamp: "2008-10-16 03:40:57 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *         building on original work by Thomas Nowotny (tnowotny@ucsd.edu)
 *
 * License: GPLv2 only
 *
 * Initial version: 2008-08-02
 *
 * CModel top cycle
 */


#include "cnlib-model.h"

#include <iostream>
#include <ctime>

using namespace std;


void
CModel::derivative( vector<double>& v, vector<double>& w)
{
      // see to this vector's dt
	w[0] = 1.;

	list<CNeuron*>::iterator  N;
	list<CSynapse*>::iterator Y;
	for_all_neurons (N)
		if ( (*N) -> is_enabled() )
			(*N) -> derivative( v, w);
	for_all_synapses (Y)
		if ( (*Y) -> is_enabled() )
			(*Y) -> derivative( v, w);
}




unsigned int
CModel::time_step( double dist, double *cpu_time_used_p)
{
	if ( _status & CN_MDL_LOGDT && _dt_logger == NULL ) {
		string	fname = name + "-dtlog";
		_dt_logger = new ofstream( fname.data());
	}

	clock_t	cpu_time_started = clock(),
		cpu_time_ended,
		cpu_time_lastchecked = cpu_time_started;

	double	time_started = model_time(),
		time_ending = time_started + dist;

	unsigned steps = 0;
	list<CNeuron* >::iterator N;
	list<CSynapse*>::iterator Y;
	do {
	      // cycle firing neurons
		for_all_neurons (N)
			if ( (*N)->is_enabled() && (*N)->is_conscious() )
				(*N) -> possibly_fire();

		if ( _status & CN_MDL_ALLOWNOISE ) {
			for_all_neurons (N)
				if ( (*N)->is_enabled() )
					(*N) -> noise();
			for_all_synapses (Y)
				if ( (*Y)->is_enabled() )
					(*Y) -> noise();
		}

	      // churn it
		_integrator->cycle();
		steps++, _cycle++;

	      // log dt
		if ( _status & CN_MDL_LOGDT && _dt_logger )
			(*_dt_logger) << model_time() << "\t" << dt() << endl;

	      // service units
		for_all_neurons (N)
			if ( (*N)->is_enabled() ) {
				if ( (*N)->_status & CN_ULISTENING )
					(*N) -> tell();
				if ( (*N)->_status & CN_RECORDINGSPIKES )
					(*N) -> spike_detect();
			}
		for_all_synapses (Y)
			if ( (*Y)->is_enabled() && (*Y)->is_listening() )
				(*Y) -> tell();

	      // entertain the user
		if ( ((double)(clock() - cpu_time_lastchecked)) / CLOCKS_PER_SEC > 3 ) {
			cpu_time_lastchecked = clock();
			printf( "\r %4.1f%%", 100 - (model_time() - time_ending) / (time_started - time_ending) * 100);
			fflush( stdout);
		}

	} while ( model_time() < time_ending );

	cpu_time_ended = clock();
	double cpu_time_taken_seconds = ((double) (cpu_time_ended - cpu_time_started)) / CLOCKS_PER_SEC;
	if ( cpu_time_used_p )
		*cpu_time_used_p = cpu_time_taken_seconds;

	cout.setf( ios::fixed);
	cout.precision( 1);
	cout << "\rRun " << _cycle << " cycles for "
	     << model_time() << "ms (avg ";
	cout.precision( 1);
	cout << model_time()/_cycle * 1e3 << "\302\265s/cyc, specific ratio to CPU time is ";
	cout.precision( 3);
	cout << cpu_time_taken_seconds * 1e3 / model_time() / _unit_cnt << ")\n";

	return steps;
}



// EOF
