// ;-*-C++-*- *  Time-stamp: "2008-10-15 16:59:51 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *         building on original work by Thomas Nowotny (tnowotny@ucsd.edu)
 *
 * License: GPLv2 only
 *
 * Initial version: 2008-10-11
 *
 * Neuron units: Hodgkin-Huxley, Entorhinal cortex stellate cell, and a friend of it
 */



#ifndef DC_NEURON_H
#define DC_NEURON_H

#include "cnlib-unit.h"


extern const double __CN_Params_NeuronHH[];
extern const char* const __CN_ParamNames_NeuronHH[];
extern const char* const __CN_ParamSyms_NeuronHH[];
extern const double __CN_Vars_NeuronHH[];
extern const char* const __CN_VarNames_NeuronHH[];
extern const char* const __CN_VarSyms_NeuronHH[];

struct SParametersNeuronHH {
	double	gNa, ENa,
		gK,  EK,
		gl, El,
		Cmem,
		Idc;
};

class CNeuronHH
  : public CNeuron {

    public:
	union {
		double i[sizeof(SParametersNeuronHH)];
		SParametersNeuronHH  n;
	} P;
	virtual double &param_value( size_t idx)	{ return P.i[idx]; }
	virtual void reset_params()	{ memcpy( P.i, __CN_DescriptorTable[NT_HH].stock_param_values, sizeof(double) * p_no()); }

      // current state
	double&  m( vector<double>& b)	{ return b[idx+1]; }
	double&  h( vector<double>& b)	{ return b[idx+2]; }
	double&  n( vector<double>& b)	{ return b[idx+3]; }
	double& dm( vector<double>& b)	{ return b[idx+1]; }
	double& dh( vector<double>& b)	{ return b[idx+2]; }
	double& dn( vector<double>& b)	{ return b[idx+3]; }

	CNeuronHH( const char *inlabel, double x, double y, double z)
	      : CNeuron( NT_HH, inlabel, x, y, z)
		{	reset_params();
		}

	virtual void derivative( vector<double>&, vector<double>&);
	virtual void possibly_fire()
		{}
	virtual void noise()
		{}
};






extern const double __CN_Params_NeuronTM[];
extern const char* const __CN_ParamNames_NeuronTM[];
extern const char* const __CN_ParamSyms_NeuronTM[];
extern const double __CN_Vars_NeuronTM[];
extern const char* const __CN_VarNames_NeuronTM[];
extern const char* const __CN_VarSyms_NeuronTM[];

struct SParametersNeuronTM
      : public SParametersNeuronHH {
	double	gKl, EKl,
		V0;
};

class CNeuronTM
  : public CNeuron {

    public:
	union {
		double i[sizeof(SParametersNeuronTM)];
		SParametersNeuronTM  n;
	} P;
	virtual double &param_value( size_t idx)	{ return P.i[idx]; }
	virtual void reset_params()
		{
			memcpy( P.i, __CN_DescriptorTable[NT_TM].stock_param_values,
				sizeof(double) * p_no());
		}

	double&  m( vector<double>& b)	{ return b[idx+1]; }
	double&  h( vector<double>& b)	{ return b[idx+2]; }
	double&  n( vector<double>& b)	{ return b[idx+3]; }
	double& dm( vector<double>& b)	{ return b[idx+1]; }
	double& dh( vector<double>& b)	{ return b[idx+2]; }
	double& dn( vector<double>& b)	{ return b[idx+3]; }

	CNeuronTM( const char *inlabel, double x, double y, double z)
	      : CNeuron( NT_TM, inlabel, x, y, z)
		{	reset_params();
		}

	virtual void derivative( vector<double>&, vector<double>&);
	virtual void possibly_fire()
		{}
	virtual void noise()
		{}
};





// Entorhinal cortex stellate cell

extern const double __CN_Params_NeuronEC[];
extern const char* const __CN_ParamNames_NeuronEC[];
extern const char* const __CN_ParamSyms_NeuronEC[];
extern const double __CN_Vars_NeuronEC[];
extern const char* const __CN_VarNames_NeuronEC[];
extern const char* const __CN_VarSyms_NeuronEC[];

struct SParametersNeuronEC
      : public SParametersNeuronTM {
	double	gh1, gh2,
		Vh;
};

class CNeuronEC
  : public CNeuron {

    public:
	union {
		double i[sizeof(SParametersNeuronEC)];
		SParametersNeuronEC  n;
	} P;
	virtual double &param_value( size_t idx)	{ return P.i[idx]; }
	virtual void reset_params()	{ memcpy( P.i, __CN_DescriptorTable[NT_EC].stock_param_values, sizeof(double) * p_no()); }

	double&    m( vector<double>& b)	{ return b[idx+1]; }
	double&    h( vector<double>& b)	{ return b[idx+2]; }
	double&    n( vector<double>& b)	{ return b[idx+3]; }
	double&  Ih1( vector<double>& b)	{ return b[idx+4]; }
	double&  Ih2( vector<double>& b)	{ return b[idx+5]; }
	double&   dm( vector<double>& b)	{ return b[idx+1]; }
	double&   dh( vector<double>& b)	{ return b[idx+2]; }
	double&   dn( vector<double>& b)	{ return b[idx+3]; }
	double& dIh1( vector<double>& b)	{ return b[idx+4]; }
	double& dIh2( vector<double>& b)	{ return b[idx+5]; }


	CNeuronEC( const char *inlabel, double x, double y, double z)
	      : CNeuron( NT_EC, inlabel, x, y, z)
		{	reset_params();
		}

	virtual void derivative( vector<double>&, vector<double>&);
	virtual void possibly_fire()
		{}
	virtual void noise()
		{}
};





extern const double __CN_Params_NeuronECA[];
extern const char* const __CN_ParamNames_NeuronECA[];
extern const char* const __CN_ParamSyms_NeuronECA[];
extern const double __CN_Vars_NeuronECA[];
extern const char* const __CN_VarNames_NeuronECA[];
extern const char* const __CN_VarSyms_NeuronECA[];

struct SParametersNeuronECA  // lacks SParametersNeuronEC's gKl and EKl, so derives directly from HH
      : public SParametersNeuronHH {
	double	gNap, gh,
		Vh;
};
class CNeuronECA
  : public CNeuron {

    public:
	union {
		double i[sizeof(SParametersNeuronECA)];
		SParametersNeuronECA  n;
	} P;
	virtual double &param_value( size_t idx)	{ return P.i[idx]; }
	virtual void reset_params()	{ memcpy( P.i, __CN_DescriptorTable[NT_ECA].stock_param_values, sizeof(double) * p_no()); }

	double&    m( vector<double>& b)	{ return b[idx+1]; }
	double&    h( vector<double>& b)	{ return b[idx+2]; }
	double&    n( vector<double>& b)	{ return b[idx+3]; }
	double& mNap( vector<double>& b)	{ return b[idx+4]; }
	double&  Ih1( vector<double>& b)	{ return b[idx+5]; }
	double&  Ih2( vector<double>& b)	{ return b[idx+6]; }

	double&    dm( vector<double>& b)	{ return b[idx+1]; }
	double&    dh( vector<double>& b)	{ return b[idx+2]; }
	double&    dn( vector<double>& b)	{ return b[idx+3]; }
	double& dmNap( vector<double>& b)	{ return b[idx+4]; }
	double&  dIh1( vector<double>& b)	{ return b[idx+5]; }
	double&  dIh2( vector<double>& b)	{ return b[idx+6]; }

	CNeuronECA( const char *inlabel, double x, double y, double z)
	      : CNeuron( NT_ECA, inlabel, x, y, z)
		{	reset_params();
		}

	virtual void derivative( vector<double>&, vector<double>&);
	virtual void possibly_fire()
		{}
	virtual void noise()
		{}
};



#endif

// EOF
