// ;-*-C++-*- *  Time-stamp: "2008-10-15 16:59:51 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *         building on original work by Thomas Nowotny (tnowotny@ucsd.edu)
 *
 * License: GPLv2 only
 *
 * Initial version: 2008-10-11
 *
 * Synapse units: Ab
 */


#ifndef CN_SYNAPSE_H
#define CN_SYNAPSE_H

#include "RandomGen.h"
#include "cnlib-unit.h"
#include "cnlib-model.h"
#include <cfloat>


extern const double __CN_Params_SynapseAb[];
extern const char* const __CN_ParamNames_SynapseAb[];
extern const char* const __CN_ParamSyms_SynapseAb[];
extern const double __CN_Vars_SynapseAb[];
extern const char* const __CN_VarNames_SynapseAb[];
extern const char* const __CN_VarSyms_SynapseAb[];

struct SParametersSynapseAb {
	double	gsyn,
		Esyn, Epre,
		asyn, bsyn,
		rtime,
		sigma;
};

class CSynapseAb
  : public CSynapse {

    public:
	union {
		double i[sizeof(SParametersSynapseAb)];
		SParametersSynapseAb n;
	} P;
	virtual double &param_value( size_t idx)	{ return P.i[idx]; }
	virtual void reset_params()
		{
			memcpy( P.i, __CN_DescriptorTable[YT_AB].stock_param_values,
				sizeof(double) * p_no());
		}

      // current state
	double&  S( vector<double>& b)	{ return b[idx+0]; }
	double& dS( vector<double>& b)	{ return b[idx+0]; }

      // ctors
	CSynapseAb( const char *inlabel,
		    CNeuron *insource, CNeuron *intarget)
	      : CSynapse( YT_AB, inlabel, insource, intarget)
		{
			reset_params();
			t_last_release_started = -INFINITY;  // prevent it from releasing at t=0
		}

      // others
	virtual double Isyn( vector<double>&b)
		{
			return -P.n.gsyn * S(b) * (_target->E(b) - P.n.Esyn);
		}

	virtual void derivative( vector<double>&, vector<double>&);

	virtual void noise()
		{
			double r = __R_std.n();
			S(*_parent_model->V) += P.n.sigma * (r/2 - r);
		}
};

#endif
