// ;-*-C++-*- *  Time-stamp: "2008-07-04 01:41:07 hmmr"
/*--------------------------------------------------------------------------
   Author: Thomas Nowotny

   Institute: Institute for Nonlinear Dynamics
	      University of California San Diego
	      La Jolla, CA 92093-0402

   email to:  tnowotny@ucsd.edu

   initial version: 2002-01-25

--------------------------------------------------------------------------*/


#ifndef SYNAPSEASTRID_H
#define SYNAPSEASTRID_H

#include "CN_synapse.h"
#include "CN_neuron.h"
#include <cmath>

#define SYNASPNO 5
#define SYNASIVARNO 1


extern double SYNAS_INIVARS[SYNASIVARNO];


class CSynapseAstrid
    : public CSynapse {

    private:
	virtual double &gsyn()		{  return p[0];  }
	virtual double &Esyn()		{  return p[1];  }
	virtual double &Epre()		{  return p[2];  }
	virtual double &delta()		{  return p[3];  }
	virtual double &kminus()	{  return p[4];  }

    public:
      // This is the constructor to be used by derived classes passing the new
      // internal var number, parameter number and type tag
	CSynapseAstrid( CNeuron *insource, CNeuron *intarget,
			double ingsyn, double inEsyn, double inEpre,
			double indelta, double inkminus,
			int inIVARNO, int inPNO, int inTYPE)
		: CSynapse( insource, intarget,
			    inIVARNO, inPNO, inTYPE)
	{    gsyn()   = ingsyn;       // gsyn strength of synapse
	     Esyn()   = inEsyn;       // Esyn reversal potential in mV
	     Epre()   = inEpre;       // Epre presyn threshold potential in mV
	     delta()  = indelta;      // delta Voltage in mV
	     kminus() = inkminus;   } // kminus rate constant

      // This is the constructor to be used directly ...
	CSynapseAstrid( CNeuron *insource, CNeuron *intarget,
			double ingsyn, double inEsyn, double inEpre,
			double indelta, double inkminus)
		: CSynapse( insource, intarget,
			    SYNASIVARNO, SYNASPNO, SYNAS)
	{    gsyn()   = ingsyn;
	     Esyn()   = inEsyn;
	     Epre()   = inEpre;
	     delta()  = indelta;
	     kminus() = inkminus;   }

	CSynapseAstrid( CNeuron *insource, CNeuron *intarget, double *inp)
		: CSynapse( insource, intarget,
			    SYNASIVARNO, SYNASPNO, SYNAS)
	{    set_p( inp);           }

	virtual ~CSynapseAstrid()		{}

	virtual double Isyn( double *x)		{    return -gsyn() * x[idx] * (target->E(x) - Esyn());  }

      // differential eqn for f, the driving variable for the pre-synaptic
      // conductance g and g itself
	virtual void derivative( double *x, double *dx)
	{
		double sinf, tau;

		sinf = 1.0 / (1.0 + exp( (Epre() - source->E(x)) / delta()));
		tau = (1.0 - sinf) / kminus();
		if ( tau < 1e-2 )
			tau = 1e-2;
		dx[idx] = (sinf - x[idx]) / tau;
	}
};

#endif


// EOF
