// ;-*-C++-*- *  Time-stamp: "2009-12-11 01:31:04 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *         building on original work by Thomas Nowotny (tnowotny@ucsd.edu)
 *
 * License: GPL
 *
 * Initial version: 2009-03-31
 *
 * Synapse units: alpha-beta
 */


#ifndef LIBCN_BASE_SYNAPSE_H
#define LIBCN_BASE_SYNAPSE_H

#include <vector>
#include <cmath>

#include "base-unit.h"


class C_BaseNeuron;
class CModel;

class C_BaseSynapse
  : public C_BaseUnit {

    friend class CModel;
    friend class C_BaseNeuron;

    protected:
	C_BaseSynapse(); // not constructible without parameters
//		{}

	C_BaseNeuron	*_source,
			*_target;

	double t_last_release_started;

    public:
	C_BaseNeuron	*source()	{  return _source;  }
	C_BaseNeuron	*target()	{  return _target;  }

    protected:
	C_BaseSynapse( TUnitType intype, const char *inlabel,
		       C_BaseNeuron *insource, C_BaseNeuron *intarget,
		       CModel *inM, int s_mask = 0);
	virtual ~C_BaseSynapse();

    public:
	double length();

	void reset_state()
		{
			C_BaseUnit::reset_state();
			t_last_release_started = -INFINITY;
		}

	virtual double Isyn() = 0;
	virtual double Isyn( vector<double>&) = 0;

	virtual void noise()
		{}

    private:
	virtual void update_queue()
		{}
};

#endif
