// ;-*-C++-*- *  Time-stamp: "2009-12-11 01:31:03 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *         building on original work by Thomas Nowotny (tnowotny@ucsd.edu)
 *
 * License: GPL
 *
 * Initial version: 2008-08-02
 *
 */



#ifndef LIBCN_STANDALONE_SYNAPSES_H
#define LIBCN_STANDALONE_SYNAPSES_H

#include "config.h"

#include <queue>
#include <iostream>
#include "base-synapse.h"
#include "base-neuron.h"
#include "standalone-attr.h"
#include "mx-attr.h"


class CModel;

class C_StandaloneSynapse
  : public C_BaseSynapse, public C_StandaloneAttributes {

    private:
	C_StandaloneSynapse();
    protected:
	C_StandaloneSynapse( TUnitType intype, const char *inlabel,
			     C_BaseNeuron* insource, C_BaseNeuron* intarget,
			     CModel* inM, int s_mask = 0);

    public:
       ~C_StandaloneSynapse();

	double &var_value( size_t v)	{ return V[v]; }
	const double &get_var_value( size_t v) const	{ return V[v]; }
	double &S()			{ return V[0]; }
	double &S( vector<double>&)	{ return V[0];  }

	void reset_vars()
		{
			memcpy( V.data(), __CNUDT[_type].stock_var_values,
				sizeof(double) * v_no());
			memcpy( V_next.data(), __CNUDT[_type].stock_var_values,
				sizeof(double) * v_no());
		}
};





class CSynapseMap
  : public C_StandaloneSynapse {

    private:
	CSynapseMap();

    public:
	CSynapseMap( const char *inlabel,
		     C_BaseNeuron* insource, C_BaseNeuron* intarget,
		     CModel *inM, int s_mask = 0, TUnitType alt_type = YT_MAP)
	      : C_StandaloneSynapse( alt_type, inlabel, insource, intarget, inM, s_mask),
		_source_was_spiking (false)
		{
			if ( !inM )
				cerr << "A Map synapse is created unattached to a model: preadvance() will cause a segfault!\n";
		}

	void preadvance();  // defined inline in model.h

	enum {
		_gsyn_, _tau_, _delta_, _Vrev_
	};
	double Isyn()
		{
			return P[_gsyn_] * S() * (_target->E() - P[_Vrev_]);
		}
	double Isyn( vector<double>& unused)
		{
			return Isyn();
		}

    protected:
	bool _source_was_spiking;
};





class CSynapseMxMap
  : public CSynapseMap, public C_MultiplexingAttributes {

    public:
	CSynapseMxMap( const char *inlabel,
				 C_BaseNeuron* insource, C_BaseNeuron* intarget,
				 CModel *inM, int s_mask = 0)
	      : CSynapseMap( inlabel, insource, intarget, inM, s_mask, YT_MXMAP)
		{
			if ( !inM )
				cerr << "A MxMap synapse is created unattached to a model: preadvance() will cause a segfault!\n";
		}

	enum {
		_gsyn_, _tau_, _delta_, _Vrev_
	};
	void preadvance();  // defined inline in model.h

    private:
	friend class CModel;
	void update_queue();
};



#endif

// EOF
