// ;-*-C++-*- *  Time-stamp: "2009-12-11 01:31:03 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *
 * License: GPL
 *
 * Initial version: 2008-12-02
 *
 * Expression parser
 */


#include <cstring>
#include <cstdlib>
#include <cstdio>

#include <iostream>
using namespace std;

#include "exprparser.h"


const char* const __exparse_error_strings[] = {
	"",
	"Empty string",
	"Unparsable value",
	"Unbalanced parentheses",
};



TExprParserError
CExpression::_do_parse( const char *str, double& parsed)
{
	if ( str == NULL ) {
		parsed = 0;
		return _status = EXPARSE_OK;
	}

	parsed = NAN;

	string	workbuf( str);
	char	*p = &workbuf[0];

	size_t ws = strspn( p, " \t");
	if ( ws )
		p += ws;
	if ( *p == '\0' )
		return _status = EXPARSE_EMPTY;

	char	*expr1 = p,
		*expr2 = NULL;
	TExprParserError subexpr_retval;

      // determine subexpressions, if any, at top level
	int level = 0;
	char *found_op_at = NULL;
	unsigned short toplevel_op_priority = 0;
#define MULTDIV		1
#define ADDSUBTRACT	2
	bool	toplevel_op_found = false,
		last_token_was_operator = true;
	while ( *p ) {
		if      ( *p == '(' )	level++;
		else if ( *p == ')' )	level--;

		if ( level < 0 )
			return _status = EXPARSE_UNBALANCED;

		if ( level == 0 ) {
			switch ( *p ) {
			case '-':
				if ( p > workbuf.c_str() && (tolower(*(p-1)) == 'e') )
					break;
				if ( last_token_was_operator ) // unary minus: skip it
					break;
			case '+':
			      // override * and /
				if ( !found_op_at ||
				     (found_op_at && toplevel_op_priority <= ADDSUBTRACT) ) {
					found_op_at = p;
					toplevel_op_found = true;
					toplevel_op_priority = ADDSUBTRACT;
				}
				last_token_was_operator = true;
			    break;
			case '*':
			case '/':
				if ( toplevel_op_priority <= MULTDIV ) { // first one found will be tlop
					found_op_at = p;
					toplevel_op_found = true;
					toplevel_op_priority = MULTDIV;
				}
				last_token_was_operator = true;
			    break;
			case ' ':
			case '\t':
			    break;
			default:
				last_token_was_operator = false;
			    break;
			}
		}

		p++;
	}
#undef ADDSUBTRACT
#undef MULTDIV

	if ( level > 0 )
		return _status = EXPARSE_UNBALANCED;

	if ( found_op_at ) {
		char op = *found_op_at;
		*found_op_at = '\0';
		expr2 = found_op_at + 1;
		double opd1, opd2;

	      // second subexpr must always be good
		subexpr_retval = _do_parse( expr2, opd2);
		if ( subexpr_retval )
			return subexpr_retval;

	      // first subexpr may be empty, but only in the case of unary minus
		subexpr_retval = _do_parse( expr1, opd1);
		if ( subexpr_retval && !(subexpr_retval == EXPARSE_EMPTY && op == '-' ) )
			return subexpr_retval;
		switch ( op ) {
		case '+':	parsed = opd1 + opd2;	break;
		case '-':	parsed = opd1 - opd2;	break;
		case '*':	parsed = opd1 * opd2;	break;
		case '/':	parsed = opd1 / opd2;	break;
		}
		return _status = EXPARSE_OK;
	}

      // single expression, possibly in parentheses
	if ( *expr1 == '(' ) {
		*strrchr( ++expr1, ')') = '\0';
		return _do_parse( expr1, parsed);
	}


	char *tailp;
	parsed = strtod( expr1, &tailp);

      // detect garbage at end
	if ( tailp && strspn( tailp, " \t\n\r") != strlen( tailp) ) {
		printf( "%s\n", expr1);
		return _status = EXPARSE_BAD;
	}

	return _status = EXPARSE_OK;
}

// EOF
