// ;-*-C++-*- *  Time-stamp: "2009-12-11 01:31:03 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *
 * License: GPL
 *
 * Initial version: 2008-11-18
 *
 */

#include <cstring>
#include <cmath>
#include <cstdlib>
#include <climits>
#include <cstdio>
#include "rangeset.h"




int
STriplet::assign( const char *parsable)
{
	const char *a = parsable,
		   *b = strchr( parsable, ':') + 1,
		   *c = strrchr( parsable, ':') + 1;
	if ( a && b && c && a != b && b != c ) {
		min = strtod( a, NULL), step = strtod( b, NULL), max = strtod( c, NULL);
		if ( min >= max || step == 0. ) {
//			fprintf( stderr, "Inconsistent values: %s\n", parsable);
			return -1;
		}
		return 0;
	} else {
//		fprintf( stderr, "Incorrectly composed triplet: %s\n", parsable);
		return -1;
	}
}







unsigned long
SRangeSet::total() const
{
	if ( ranges.size() == 0 )
		return 0;

	double result_dbl = 1.;
	for ( size_t i = 0; i < ranges.size(); i++ )
		result_dbl *= (ranges[i].max - ranges[i].min) / ranges[i].step + 1;
	return (result_dbl > (double)ULONG_MAX) ?ULONG_MAX :(unsigned long)result_dbl;
}




const char *__cnaux_rangeset_default_fname_prefix = "run";

const char* const
SRangeSet::generate_fname_first( const char *prefix)
{
	filename = prefix;
	char _[16];
	for ( size_t i = 0; i < vars.size(); i++ ) {
		vars[i] = ranges[i].min;
		snprintf( _, 15, "%s%9.7f", i ?"-" :"", vars[i]);
		filename += _;
	}
	_iteration = 0;
	return filename.data();
}



const char* const
SRangeSet::generate_fname_current( const char *prefix)
{
	filename = prefix;
	char _[16];
	for ( size_t i = 0; i < vars.size(); i++ ) {
		snprintf( _, 15, "%s%9.7f", i ?"-" :"", vars[i]);
		filename += _;
	}
	return filename.data();
}



const char* const
SRangeSet::_do_generate_fname_next( size_t l, const char *prefix)
{
	if ( vars[l] + ranges[l].step <= ranges[l].max ) {
		vars[l] += ranges[l].step;
		_iteration++;

		filename = prefix;
		char _[16];
		for ( size_t i = 0; i < vars.size(); i++ ) {
			snprintf( _, 15, "%s%9.7f", i ?"-" :"", vars[i]);
			filename += _;
		}
		return filename.data();

	} else if ( l > 0 ) {
		vars[l] = ranges[l].min;
		return _do_generate_fname_next( l - 1, prefix);

	} else
		return NULL;
}





size_t
SRangeSet::get_n_ranges_from_template()
{
	size_t n = 0;
	char token[6];
	do
		snprintf( token, 5, "%%%zd", ++n);
	while ( strstr( templatto.c_str(), token) );
	n--;
	vars.resize( n);
	ranges.resize( n);
	return n;
}



char	__cn_mark_char1 = '%',
	__cn_mark_char2 = '%',
	__cn_mark_char2_serial = '#';

unsigned __cnaux_rangeset_default_width = 12;


#define RANGESET_MAX_FILLOUT_FIELD_SIZE 40

const char* const
SRangeSet::fillout()
{
	if ( templatto.size() == 0 )
		return NULL;

	content = templatto;

	size_t pos;
	char	buf[RANGESET_MAX_FILLOUT_FIELD_SIZE];
	char	token[4] = { __cn_mark_char1, __cn_mark_char2_serial, '\0' };

	snprintf( buf, RANGESET_MAX_FILLOUT_FIELD_SIZE, "%05lu", _iteration);
	while ( (pos = content.find( token)) != content.npos )
		content.replace( pos, 2, buf);

	for ( size_t i = 0; i < vars.size(); i++ ) {
	      // substitute %n with var[n]
		token[0] = __cn_mark_char1;
		token[1] = (char)i + '1',   token[2] = '\0';
		snprintf( buf, RANGESET_MAX_FILLOUT_FIELD_SIZE,
			  "%.*g", width, vars[i]);
		while ( (pos = content.find( token)) != content.npos )
			content.replace( pos, 2, buf);

	      // substitute %% with <var[0]>-<var[1]>-...-<var[last]>, as in generate_next
		token[0] = __cn_mark_char1, token[1] = __cn_mark_char1;
		token[2] = '\0';
		string __ = "";
		for ( unsigned v = 0; v < vars.size(); v++ ) {
			snprintf( buf, RANGESET_MAX_FILLOUT_FIELD_SIZE, "%s%9.7f", v ?"-" :"", vars[v]);
			__ += buf;
		}
		while ( (pos = content.find( token)) != content.npos )
			content.replace( pos, 2, __);
	}

	return content.c_str();
}



// EOF
