// ;-*-C++-*- *  Time-stamp: "2009-12-11 01:31:03 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *
 * License: GPL
 *
 * Initial version: 2008-11-18
 *
 */

#ifndef CNAUX_RANGESET_H
#define CNAUX_RANGESET_H

#include <string>
#include <vector>
#include <cstring>

using namespace std;

struct STriplet {
	double min, step, max;
	int assign( const char *parsable);
};


extern const char *__cnaux_rangeset_default_fname_prefix;

extern unsigned __cnaux_rangeset_default_width;

struct SRangeSet {

    public:
	vector<STriplet> ranges;
	vector<double> vars;

	string templatto;

	unsigned width;

	size_t n_ranges() const
		{ return ranges.size(); }

	void add_var( STriplet& T)
		{ ranges.push_back( T), vars.push_back( (T.max + T.min)/2); }
	SRangeSet& operator << ( STriplet& T)
		{ add_var( T); return *this; }

	int assign_vars( const vector<double> &rv)
		{ if ( vars.size() == rv.size() )  { vars = rv; return 0; } else return -1; }
	void assign_vars( const double *rv)
		{ memcpy( vars.data(), rv, sizeof(double) * vars.size()); }
	size_t get_n_ranges_from_template();


	unsigned long current_iteration() const
		{ return _iteration; }
	unsigned long total() const;

	SRangeSet()
		{	_iteration = 0, width = __cnaux_rangeset_default_width; }
	SRangeSet( const char *templatto_in)
		{	_iteration = 0, width = __cnaux_rangeset_default_width; templatto = templatto_in; }
	SRangeSet( const SRangeSet& rv)
		{	ranges = rv.ranges; vars = rv.vars; templatto = rv.templatto;
			width = rv.width, _iteration = rv._iteration;			}

	const char* const generate_fname_first( const char *prefix = __cnaux_rangeset_default_fname_prefix);
	const char* const generate_fname_current( const char *prefix = __cnaux_rangeset_default_fname_prefix);
	const char* const generate_fname_next( const char *prefix = __cnaux_rangeset_default_fname_prefix)
		{ return vars.size() ?_do_generate_fname_next( vars.size()-1, prefix) :"(uninitialized rangeset)"; }

	const char* const fillout();

	unsigned long _iteration;
	string content, filename;

	const char* const _do_generate_fname_next( size_t l, const char *prefix);
};

extern char __cn_mark_char1, __cn_mark_char2, __cn_mark_char2_serial;




#endif

// EOF
