// ;-*-C++-*- *  Time-stamp: "2010-01-21 21:20:57 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *
 * License: GPL
 *
 * Initial version: 2008-11-04
 *
 * CNModel runner
 */

#ifndef CN_RUNNER_H
#define CN_RUNNER_H

#include <vector>
#include <iostream>
#include "exprparser.h"

#include "model.h"
#include "integrate-rk65.h"
#include "base-unit.h"




struct SCNRunnerOptions {
	unsigned long
		running_time;
	bool	listen_1varonly:1,
		listen_deferwrite:1,
		dump_params:1,
		list_units:1,
		sort_units:1,
		log_dt:1,
		log_spikers:1,
		log_spikers_use_serial_id:1,
		log_sdf:1,
		allow_silent_run:1,
		display_progress_percent:1,
		display_progress_time:1;
	const char *working_dir /*,
				 *spike_cnt_stats_file ,
				 *single_neuron */;
	unsigned precision;
	double	spike_threshold,
		spike_lapse,
		logging_dt,
		discrete_dt,
		dt_max,
		dt_min,
		dt_max_fac;
	unsigned
		sxf_sample,
		sdf_sigma;
	int	verbosely;

	SCNRunnerOptions()
	      : running_time (0.),
		listen_1varonly (false), listen_deferwrite (false),
		dump_params(false), list_units (false),
		sort_units (false),
		log_dt (false),
		log_spikers (false), log_spikers_use_serial_id (false),
		log_sdf (false),
		allow_silent_run (false),
		display_progress_percent (true),
		display_progress_time (false),
		working_dir ("."),
		precision (8),
		spike_threshold (0.), spike_lapse (3.),
		logging_dt(1.),
		discrete_dt(.5),
		dt_max (.5), dt_min (1e-5), dt_max_fac (5.),
		sxf_sample (0.), sdf_sigma (0.),
		verbosely (1)
		{}
};

extern vector<string> ParameterFiles;

extern SCNRunnerOptions Options;

//extern const char *default_nml_fname;

extern vector<string> Listeners;
extern vector<string> CmdLineUnitSpecs;
extern vector<string> Spikeloggers;
extern vector<string> ToRemove;

extern vector<CModel::SDecimationGroup> ToDecimate;



int do_unit_specs();
int process_tags();
int assign_unit_static_params();
void assign_unit_current_range_values();


struct SParm {
	C_BaseUnit *unit;
	unsigned short parm;
	C_BaseUnit::TDeviceAssignee kind;
};

struct SParmValuePair : SParm {
	double	value;
};
struct SParmReaderOrScheduler : SParm {
	string  fname;
	bool	is_looping;
};
struct SParmValueRange : SParmValuePair {
	double	min, span, max;
};
struct SParmValueBatch : SParmValuePair {
	double	min, step, max;
};

extern vector<SParmValuePair> Singles;
extern vector<SParmValueRange> Ranges;
extern vector<SParmReaderOrScheduler> Readers;
extern vector<SParmReaderOrScheduler> Schedulers;

extern vector<SParmValueBatch> Batches;

int do_param_batch( unsigned);



extern CModel *Model;

int prepare_model();



void dump_model_units();

//void report_spike_count_stats();




#endif

// EOF
