// ;-*-C++-*- *  Time-stamp: "2009-12-11 01:31:05 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *         building on original work by Thomas Nowotny (tnowotny@ucsd.edu)
 *
 * License: GPL
 *
 * Initial version: 2008-11-11
 *
 * A remedy against forgetting to pass -d to cnrun
 */


#include <iostream>
#include <fstream>
#include <vector>
#include <limits>
#include <cstdlib>
#include <cstring>
#include <cmath>

using namespace std;

int
main( int argc, char *argv[])
{
	if ( argc != 4 ) {
		cerr << "Expecting <fname> <period> <sigma>,\n" "got " << argc-1 << " argument(s)\n";
		//return -1;
	}

	string fname( argv[1]);

	double	sxf_sample = strtod( argv[2], NULL),
		sdf_sigma = strtod( argv[3], NULL);


	ifstream is( fname.data());
	if ( !is.good() ) {
		cerr << "Can't read from file " << fname << endl;
		return -1;
	}
	is.ignore( numeric_limits<streamsize>::max(), '\n');

	if ( fname.rfind( ".spikes") == fname.size() - 7 )
		fname.erase( fname.size() - 7, fname.size());
	fname += ".sdf";

	ofstream os( fname.data());
	if ( !os.good() ) {
		cerr << "Can't open " << fname << " for writing\n";
		return -1;
	}
	os << "#<t>\t<sdf>\t<nspikes>\n";


	vector<double> _spike_history;
	while ( true ) {
		double datum;
		is >> datum;
		if ( is.eof() )
			break;
		_spike_history.push_back( datum);
	}

	double	at, len = _spike_history.back(), dt,
		sdf_var = sdf_sigma * sdf_sigma;
	cout << fname << ": " << _spike_history.size() << " spikes (last at " << _spike_history.back() << ")\n";
	for ( at = sxf_sample; at < len; at += sxf_sample ) {
		double result = 0.;
		unsigned nspikes = 0;
		for ( vector<double>::iterator T = _spike_history.begin(); T != _spike_history.end(); T++ ) {

			dt = *T - at;
			if ( dt < -sxf_sample/2. )
				continue;
			if ( dt >  sxf_sample/2. )
				break;

			nspikes++;
			result += exp( -dt*dt/sdf_var);

		}
		os << at << "\t" << result << "\t" << nspikes << endl;
	}

	return 0;
}

// EOF
