// ;-*-C++-*- *  Time-stamp: "2008-10-14 15:05:00 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *         building on original work by Thomas Nowotny (tnowotny@ucsd.edu)
 *
 * License: GPLv2 only
 *
 * Initial version: 2008-08-02
 *
 * A `neuron' that reads its state from a file
 * the individual units
 */


#include <fstream>
#include <iostream>
#include "cnlib-reader-tape.h"
#include "cnlib-model.h"

using namespace std;


const double __CN_Vars_NeuronReader[] = {
	-20.
};
const char* const __CN_VarNames_NeuronReader[] = {
	"Membrane potential"
};
const char* const __CN_VarSyms_NeuronReader[] = {
	"E"
};




#define ALLOC_CHUNK 20

CNReader::CNReader( const char *inlabel,
		    const char *fname,
		    double intimestep,
		    int instatus)
      : CNeuron( NT_READER, inlabel, 0., 0., 0.)
{
	_timestep = intimestep;
	_status |= (CN_NCONSCIOUS | instatus);

	try {
		ifstream is;
		is.open( fname, ios::in);

		if ( !is.good() )
			throw ( ios::failure( "open error") );

		if ( _timestep == 0. )
			is >> _timestep;

		double datum;
		while ( !is.eof() ) {
			if ( sizeof Et % ALLOC_CHUNK == 0 )
				Et.reserve( Et.size() + ALLOC_CHUNK);
			is >> datum;
			Et.push_back( datum);
		}

		cout << "CNReader \"" << label << "\": "
		     << Et.size()-1 << " E value(s) at timestep " << _timestep
		     << " from file " << fname << endl;

	} catch ( exception& xe ) {
		cerr << "Failed to open CNReader data file " << fname /* << ": " << xe.what() */ << endl;
		enable( false);
	}

	if ( is_enabled() )
		next_state_E = Et.front();
	// don't care whether next_state_E has apples or oranges
}


#undef ALLOC_CHUNK



void
CNReader::possibly_fire()
{
	E() = next_state_E;

	size_t	i_abs = (size_t)(model_time() / _timestep),
		i_eff = (_status & CN_SMPLTAPE_LOOP)
		? i_abs % Et.size()
		: min (i_abs, Et.size() - 1);

	next_state_E = Et[ i_eff ];
//	cerr << "[" << i_eff << "]" << next_state_E;
}

// EOF
