// ;-*-C++-*- *  Time-stamp: "2008-10-16 03:36:49 hmmr"
/*--------------------------------------------------------------------------
   Author: Thomas Nowotny

   Institute: Institute for Nonlinear Dynamics
	      University of California San Diego
	      La Jolla, CA 92093-0402

   email to:  tnowotny@ucsd.edu

   initial version: 2005-08-17

--------------------------------------------------------------------------*/


#include "cnlib-synapse.h"
#include "cnlib-model.h"


const double __CN_Params_SynapseAb[] = {
	0.5,
	0.0,
      -20.0,
	0.1,
	0.2,
	5.0,
	0.  // taken from CNtest
};
const char* const __CN_ParamNames_SynapseAb[] = {
	"Strength of synapse",
	"Reversal potential",
	"Presyn threshold potential",
	"Rise rate, 1/msec",
	"Decay rate, 1/msec",
	"Time of transmitter release",
	"Noise level",
};
const char* const __CN_ParamSyms_SynapseAb[] = {
	"gsyn",
	"Esyn",
	"Epre",
	"\316\261",
	"\316\262",
	"trel",
	"\317\203",
};

const double __CN_Vars_SynapseAb[] = {
	0.
};
const char* const __CN_VarNames_SynapseAb[] = {
	"State variable"
};
const char* const __CN_VarSyms_SynapseAb[] = {
	"S"
};


void
CSynapseAb::derivative( vector<double>& x, vector<double>& dx)
{
	double time_releasing = x[0] - t_last_release_started;

	if ( time_releasing <= P.n.rtime ) {
	      // continue release from an old spike
		dS(dx) = P.n.asyn * (1.0 - S(x)) - P.n.bsyn * S(x);
	} else
		if ( _source->E(x) > P.n.Epre && time_releasing > P.n.rtime ) {
		      // new spike ... start releasing
			t_last_release_started = x[0];
			dS(dx) = P.n.asyn * (1.0 - S(x)) - P.n.bsyn * S(x);
		} else {
		      // no release
			dS(dx) = -P.n.bsyn * S(x);
		}
}

// EOF
